@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2020, Manfred Moitzi
+# Copyright (c) 2018-2021, Manfred Moitzi
 # License: MIT License
 from typing import Tuple, List, Any, Iterable, Sequence, TYPE_CHECKING
 from functools import partial
@@ -8,7 +8,8 @@ import random
 if TYPE_CHECKING:
     from ezdxf.eztypes import VecXY, Vertex
 
-isclose = partial(math.isclose, abs_tol=1e-12)
+ABS_TOL = 1e-12
+isclose = partial(math.isclose, abs_tol=ABS_TOL)
 
 
 class Vec3:
@@ -261,16 +262,21 @@ class Vec3:
 
     @property
     def is_null(self) -> bool:
-        """ ``True`` for ``Vec3(0, 0, 0)``. """
-        return isclose(self._x, 0.) and \
-               isclose(self._y, 0.) and \
-               isclose(self._z, 0.)
+        """ ``True`` if all components are close to zero: ``Vec3(0, 0, 0)``.
+        Has a fixed absolute testing tolerance of 1e-12!
+        """
+        return abs(self._x) <= ABS_TOL and \
+               abs(self._y) <= ABS_TOL and \
+               abs(self._z) <= ABS_TOL
 
-    def is_parallel(self, other: 'Vec3', abs_tol: float = 1e-12) -> bool:
+    def is_parallel(self, other: 'Vec3', *, rel_tol: float = 1e-9,
+                    abs_tol: float = 1e-12) -> bool:
         """ Returns ``True`` if `self` and `other` are parallel to vectors. """
         v1 = self.normalize()
         v2 = other.normalize()
-        return v1.isclose(v2, abs_tol) or v1.isclose(-v2, abs_tol)
+        return v1.isclose(
+            v2, rel_tol=rel_tol, abs_tol=abs_tol) or v1.isclose(
+            -v2, rel_tol=rel_tol, abs_tol=abs_tol)
 
     @property
     def spatial_angle(self) -> float:
@@ -336,12 +342,19 @@ class Vec3:
         """ Returns ``True`` if vector is not ``(0, 0, 0)``. """
         return not self.is_null
 
-    def isclose(self, other: 'Vertex', abs_tol: float = 1e-12) -> bool:
-        """ Returns ``True`` if `self` is close to `other`. Uses :func:`math.isclose` to compare all axis. """
+    def isclose(self, other: 'Vertex', *, rel_tol: float = 1e-9,
+                abs_tol: float = 1e-12) -> bool:
+        """ Returns ``True`` if `self` is close to `other`.
+        Uses :func:`math.isclose` to compare all axis.
+
+        Learn more about the :func:`math.isclose` function in
+        `PEP 485 <https://www.python.org/dev/peps/pep-0485/>`_.
+
+        """
         x, y, z = self.decompose(other)
-        return math.isclose(self._x, x, abs_tol=abs_tol) and \
-               math.isclose(self._y, y, abs_tol=abs_tol) and \
-               math.isclose(self._z, z, abs_tol=abs_tol)
+        return math.isclose(self._x, x, rel_tol=rel_tol, abs_tol=abs_tol) and \
+               math.isclose(self._y, y, rel_tol=rel_tol, abs_tol=abs_tol) and \
+               math.isclose(self._z, z, rel_tol=rel_tol, abs_tol=abs_tol)
 
     def __eq__(self, other: 'Vertex') -> bool:
         """
@@ -350,10 +363,9 @@ class Vec3:
         Args:
             other: :class:`Vec3` compatible object
         """
-        x, y, z = self.decompose(other)
-        return isclose(self._x, x) and \
-               isclose(self._y, y) and \
-               isclose(self._z, z)
+        if not isinstance(other, Vec3):
+            other = Vec3(other)
+        return self.x == other.x and self.y == other.y and self.z == other.z
 
     def __lt__(self, other: 'Vertex') -> bool:
         """
@@ -653,7 +665,7 @@ class Vec2:
 
     @property
     def is_null(self) -> bool:
-        return isclose(self.x, 0.) and isclose(self.y, 0.)
+        return abs(self.x) <= ABS_TOL and abs(self.y) <= ABS_TOL
 
     @property
     def angle(self) -> float:
@@ -718,15 +730,18 @@ class Vec2:
     def __bool__(self) -> bool:
         return not self.is_null
 
-    def isclose(self, other: 'VecXY', abs_tol: float = 1e-12) -> bool:
-        return math.isclose(self.x, other.x, abs_tol=abs_tol) and math.isclose(
-            self.y, other.y, abs_tol=abs_tol)
+    def isclose(self, other: 'VecXY', *, rel_tol: float = 1e-9,
+                abs_tol: float = 1e-12) -> bool:
+        if not isinstance(other, Vec2):
+            other = Vec2(other)
+        return math.isclose(
+            self.x, other.x, rel_tol=rel_tol, abs_tol=abs_tol) and math.isclose(
+            self.y, other.y, rel_tol=rel_tol, abs_tol=abs_tol)
 
     def __eq__(self, other: 'Vertex') -> bool:
-        # accepts also tuples, for more convenience at testing
-        x, y, *_ = other
-        return isclose(self.x, x) and \
-               isclose(self.y, y)
+        if not isinstance(other, Vec2):
+            other = Vec2(other)
+        return self.x == other.x and self.y == other.y
 
     def __lt__(self, other: 'Vertex') -> bool:
         # accepts also tuples, for more convenience at testing
