@@ -5,7 +5,7 @@ import abc
 import math
 import itertools
 import enum
-from ezdxf.math import Matrix44
+from ezdxf.math import Matrix44, BoundingBox2d
 
 """
 
@@ -249,6 +249,18 @@ class Box(abc.ABC):
         """ Render content at the final location. """
         pass
 
+    def bbox(self) -> BoundingBox2d:
+        """ Returns the 2D bounding box of the container. If the cell is
+        not placed the top/left corner = (0, 0).
+
+        """
+        try:
+            x, y = self.final_location()
+        except (ValueError, TypeError):
+            x, y = 0, 0
+        return BoundingBox2d([
+            (x, y), (x + self.total_width, y - self.total_height)])
+
 
 class Cell(Box):  # ABC
     is_visible = False
@@ -310,6 +322,10 @@ class NonBreakingSpace(Glue):
     pass
 
 
+class Tabulator(Glue):
+    pass
+
+
 class CellAlignment(enum.IntEnum):
     BOTTOM = 0
     CENTER = 1
@@ -557,9 +573,9 @@ class Fraction(ContentCell):
         self.renderer.line(x1, y1, x2, y2, m)
 
 
-_content = {Text, Fraction}
-_glue = {Space, NonBreakingSpace}
-_no_break = {Text, Fraction, NonBreakingSpace}
+_content = (Text, Fraction)
+_glue = (Space, NonBreakingSpace, Tabulator)
+_no_break = (Text, Fraction, NonBreakingSpace)
 
 
 def normalize_cells(cells: Iterable[Cell]) -> List[Cell]:
@@ -567,7 +583,7 @@ def normalize_cells(cells: Iterable[Cell]) -> List[Cell]:
         index = len(content) - 1
         while index >= 0:
             cell = content[index]
-            if type(cell) is NonBreakingSpace:
+            if isinstance(cell, NonBreakingSpace):
                 content[index] = cell.to_space()
                 index -= 1
             else:
@@ -575,10 +591,10 @@ def normalize_cells(cells: Iterable[Cell]) -> List[Cell]:
 
     def is_useless_nbsp():
         try:
-            peek = type(cells[index + 1])
+            peek = cells[index + 1]
         except IndexError:
             return True
-        if prev not in _no_break or peek not in _no_break:
+        if not isinstance(prev, _no_break) or not isinstance(peek, _no_break):
             return True
         return False
 
@@ -586,20 +602,19 @@ def normalize_cells(cells: Iterable[Cell]) -> List[Cell]:
     cells = list(cells)
     prev = None
     for index, cell in enumerate(cells):
-        current = type(cell)
-        if current in _content:
-            if prev in _content:
+
+        if isinstance(cell, _content):
+            if isinstance(prev, _content):
                 raise ValueError('no glue between content cells')
-        elif current is NonBreakingSpace and is_useless_nbsp():
+        elif isinstance(cell, NonBreakingSpace) and is_useless_nbsp():
             cell = cell.to_space()
-            current = type(cell)
             replace_pending_nbsp_by_spaces()
 
-        prev = current
+        prev = cell
         content.append(cell)
 
     # remove pending glue:
-    while content and (type(content[-1]) in _glue):
+    while content and isinstance(content[-1], _glue):
         content.pop()
 
     return content
@@ -818,6 +833,30 @@ class Paragraph(Container):  # ABC
         pass
 
 
+class EmptyParagraph(Cell):
+    """ Spacer between two paragraphs, represents empty lines like in
+    "line1\n\nline2".
+    """
+
+    def __init__(self, cap_height: float, line_spacing: float = 1):
+        self._height = leading(cap_height, line_spacing)
+        self._width = 0
+
+    @property
+    def total_width(self) -> float:
+        return self._width
+
+    @property
+    def total_height(self) -> float:
+        return self._height
+
+    def set_total_width(self, width: float):
+        self._width = width
+
+    def distribute_content(self, height: float = None):
+        pass
+
+
 class FlowTextAlignment(enum.IntEnum):
     DEFAULT = 0
     LEFT = 1
@@ -964,11 +1003,11 @@ class FlowText(Paragraph):
             """
             group = []
             if cells:
-                next_t = type(cells[-1])
-                if next_t is NonBreakingSpace:
+                next_t = cells[-1]
+                if isinstance(next_t, NonBreakingSpace):
                     group.append(cells.pop())
                     group.extend(next_group(cells))
-                elif next_t in _content:
+                elif isinstance(next_t, _content):
                     group.append(cells.pop())
                     if cells and isinstance(cells[-1], NonBreakingSpace):
                         group.extend(next_group(cells))
@@ -1085,14 +1124,11 @@ def leading(cap_height: float, line_spacing: float = 1.0) -> float:
         line_spacing: line spacing factor as percentage of 3-on-5 spacing
 
     """
-    # 3-on-5 line spacing = 5/3 = 1.667
+    # method "exact": 3-on-5 line spacing = 5/3 = 1.667
+    # method "at least" is not supported
     return cap_height * 1.667 * line_spacing
 
 
-class BulletList(Paragraph):
-    pass
-
-
 class Column(Container):
     def __init__(self, width: float,
                  height: float = None,
@@ -1191,6 +1227,10 @@ class Layout(Container):
     def __len__(self):
         return len(self._columns)
 
+    @property
+    def current_column_index(self):
+        return self._current_column
+
     @property
     def content_width(self):
         width = self._content_width
@@ -1296,3 +1336,8 @@ class Layout(Container):
         empty = self._columns[-1].clone_empty()
         self._columns.append(empty)
         return empty
+
+    def next_column(self) -> None:
+        self._current_column += 1
+        if self._current_column >= len(self._columns):
+            self._new_column()
